/** \file
	Program szyfrujco - deszyfrujcy (algorytm symetryczny) dla plikw binarnych
*/

#include <vcl.h>
#include <stdio.h>
#include <stdlib.h>
#pragma hdrstop

#include "mainUnit.h"
#include <algorithm>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "CGAUGES"
#pragma resource "*.dfm"
TfrmMain *frmMain;
unsigned int lSize = 0;							///< rozmiar pliku wczytanego/wyjciowego


//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
}



/**
    Crypt routine is taken from article
    http://www.codeproject.com/KB/security/crypt_routine.aspx?fid=4418&df=90&mpp=25&noise=3&sort=Position&view=Quick&select=943809&fr=1
    by Shrishail Rana. It may contain some nonsignificant modifications and has some ugly VCL progress bar update added.
    
    This Crypt routine uses an algorithm created by Rodney Thayer.
    This algorithm can use keys of various sizes.
    With a 40-bit key (5 characters) it can be freely exported from the U.S.
    The cipher is considered robust with 128 bits of key material but can use up to 2048 bits.
    It is compatible with RSAs RC4 algorithm.
*/
void TfrmMain::CryptInit(char* key)
{
	i = j = k = t =  x = 0;
	temp = 0;

    //always initialize the arrays with zero
    ZeroMemory(Sbox, sizeof(Sbox));
    ZeroMemory(Sbox2, sizeof(Sbox2));

    //initialize sbox i
    for(i = 0; i < 256U; i++)
    {
        Sbox[i] = (char)i;
    }

    j = 0;

    //initialize the sbox2 with user key
    for(i = 0; i < 256U ; i++)
    {
        if(j == strlen(key))
        {
            j = 0;
        }
        Sbox2[i] = key[j++];
    }


    j = 0 ; //Initialize j
    //scramble sbox1 with sbox2
    for(i = 0; i < 256; i++)
    {
        j = (j + (unsigned long) Sbox[i] + (unsigned long) Sbox2[i]) % 256U ;
        temp =  Sbox[i];
        Sbox[i] = Sbox[j];
        Sbox[j] =  temp;
    }

	i = j = 0;
}

void TfrmMain::CryptProcess(char *inp, unsigned int inplen)
{
    for(x = 0; x < inplen; x++)
    {
        //increment i
        i = (i + 1U) % 256U;
        //increment j
        j = (j + (unsigned long) Sbox[i]) % 256U;

        //Scramble SBox #1 further so encryption routine will
        //will repeat itself at great interval
        temp = Sbox[i];
        Sbox[i] = Sbox[j] ;
        Sbox[j] = temp;

        //Get ready to create pseudo random  byte for encryption key
        t = ((unsigned long) Sbox[i] + (unsigned long) Sbox[j]) %  256U ;

        //get the random byte
        k = Sbox[t];

        //xor with the data and done
		inp[x] = (inp[x] ^ k);
	}
}

//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnOpenClick(TObject *Sender)
{
	int skipHeader = StrToIntDef(edFileHeader->Text, -1);
	if (skipHeader < 0)
	{
		Application->MessageBox("Invalid number of bytes to skip entered", frmMain->Caption.c_str(), MB_ICONEXCLAMATION);
		return;
	}
	btnOpen->Enabled = false;
	progress->Position = 0;
	CryptInit(edPwd->Text.c_str());
	if(OpenDialog1->Execute())
	{
		FILE *pFileIn = fopen(OpenDialog1->FileName.c_str(), "rb");
		FILE *pFileOut = NULL;
		if(!pFileIn)
		{
			Application->MessageBox("Failed to open file", frmMain->Caption.c_str(), MB_ICONEXCLAMATION);
			btnOpen->Enabled = true;
			return;
		}

        fseek (pFileIn , 0 , SEEK_END);
        unsigned long lSize = ftell (pFileIn);
		rewind (pFileIn);

        if(SaveDialog1->Execute())
		{
			pFileOut = fopen(SaveDialog1->FileName.c_str(), "wb");	//file will be overwritten
		}

		if(pFileOut == NULL)
		{
			Application->MessageBox("Failed to create output file", frmMain->Caption.c_str(), MB_ICONEXCLAMATION);
		}
		else
		{
			char buffer[100 * 1024];
			unsigned int cnt, expected;

			do
			{
				expected = std::min<int>(skipHeader, sizeof(buffer));
				cnt = fread(buffer, 1, expected, pFileIn);
				if (fwrite(buffer, 1, cnt, pFileOut) != cnt)
				{
					Application->MessageBox("Write error!", frmMain->Caption.c_str(), MB_ICONEXCLAMATION);
					cnt = 0;
				}
				else
				{
                	skipHeader -= cnt;
				}
			}
			while (cnt == expected && skipHeader > 0);

			if (cnt > 0)
			{
				do
				{
					Application->ProcessMessages();	//pozwl na przesuwanie okna itp.
					frmMain->progress->Position = 100 * (float)ftell(pFileIn)/(float)lSize;
					frmMain->Update();

					cnt = fread(buffer, 1, sizeof(buffer), pFileIn);
					CryptProcess(buffer, cnt);
					if (fwrite(buffer, 1, cnt, pFileOut) != cnt)
					{
						Application->MessageBox("Write error!", frmMain->Caption.c_str(), MB_ICONEXCLAMATION);
						cnt = 0;
					}
				}
				while (cnt == sizeof(buffer));
				frmMain->progress->Position = 100;
			}
		}

		if(pFileIn) fclose (pFileIn);
		if(pFileOut) fclose (pFileOut);
	}
	btnOpen->Enabled = true;
}
//---------------------------------------------------------------------------
